#!/usr/bin/env python3
# vajab sage'i (https://sagemath.org)
from sage.all import *
import random

seed = " ".join(sys.argv[1:])
random.seed("coppersmith" + seed)
set_random_seed(random.getrandbits(127)) # sets up all sage RNGs

def output_graph(G):
    assert G.is_connected()
    verts = G.vertices()
    random.shuffle(verts)
    renumbering = {x: i+1 for i,x in enumerate(verts)}
    print(len(verts), G.num_edges())
    edgelist = []
    for u,v in G.edges(labels=False):
        edgelist.append((renumbering[u], renumbering[v]))
    random.shuffle(edgelist)
    for x in edgelist: print(*x)

def bipartite_n_m(n1, n2, m, marker=0):
    G = Graph()
    G.add_vertices([(0, i, marker) for i in range(n1)])
    G.add_vertices([(1, i, marker) for i in range(n2)])
    for _ in range(m):
        while True:
            a = (0, random.randrange(n1), marker)
            b = (1, random.randrange(n2), marker)
            if G.has_edge(a, b): # at most 50% fail chance for reasonable inputs
                continue
            G.add_edge(a, b)
            break
    return G

def connect_components(G):
    comps = G.connected_components(sort=False)
    while len(comps) > 1:
        ca = random.randrange(len(comps))
        cb = ca
        while cb == ca: cb = random.randrange(len(comps))
        compa = comps[ca]
        compb = comps[cb]
        joina = random.choice(compa)
        joinb = random.choice(compb)
        # hack, means this only works with bipartite_n_m:
        # make sure we keep the bipartite structure so the markers are accurate
        if joina[0] == joinb[0]: continue
        G.add_edge(joina, joinb)
        compa += compb
        del comps[cb]

kind = sys.argv[1]

if kind == "bipartite":
    n1 = int(sys.argv[2])
    n2 = int(sys.argv[3])
    edges = int(sys.argv[4])
    G = bipartite_n_m(n1, n2, edges)
    #print(G.connected_components_number())
    connect_components(G)
    output_graph(G)

elif kind == "cycle":
    n = int(sys.argv[2])
    output_graph(graphs.CycleGraph(n))

elif kind == "weirdcycle":
    n = int(sys.argv[2])
    r""" midagi umbes sellise kujuga:
         \      /- 0 -\ /- 0 -\ /-
      ... 2 -- 2       2       2    ...
         /      \- 1 -/ \- 1 -/ \-  """
    G = Graph()
    G.add_vertex((0, 2))
    for i in range(1, n):
        G.add_vertices([(i, 0), (i, 1), (i, 2)])
        G.add_edge((i-1, 2), (i, 0))
        G.add_edge((i-1, 2), (i, 1))
        G.add_edge((i, 0), (i, 2))
        G.add_edge((i, 1), (i, 2))
    G.add_edge((n-1, 2), (0, 2))
    output_graph(G)

elif kind == "unicycle":
    n1, n2, edges, n_cyc = map(int, sys.argv[2:])
    
    G = bipartite_n_m(n1, n2, edges, 0)
    connect_components(G)

    cyc = graphs.CycleGraph(n_cyc)
    G = G.union(cyc)
    G.add_edge((0, 0, 0), 0)
    output_graph(G)

elif kind == "bicycle":
    n1, n2, edges, n_cyc, n2_cyc = map(int, sys.argv[2:])
    
    G = bipartite_n_m(n1, n2, edges, 0)
    connect_components(G)

    cyc1 = graphs.CycleGraph(n_cyc)
    cyc2 = graphs.CycleGraph(n2_cyc)
    # join the 2 cycles into G at random spots
    G2 = G.union(cyc1.disjoint_union(cyc2))
    G2.add_edge((0, 0, 0), (0, 0))
    G2.add_edge(G.random_vertex(), (1, 0))
    output_graph(G2)

elif kind == "bicycle2":
    # boring shape, but stress-tests output size hopefully...
    n1, n2 = map(int, sys.argv[2:])
    G1 = graphs.CycleGraph(n1)
    G2 = graphs.CycleGraph(n2)
    G = G1.disjoint_union(G2)
    G.add_edge((0, 0), (1, 0))
    output_graph(G)

elif kind == "cone":
    # https://en.wikipedia.org/wiki/Mycielskian#Cones_over_graphs
    # guarantees quite high odd girth and high chromatic number
    G = graphs.PathGraph(2)
    deltas = map(int, sys.argv[2].split(','))
    for i in deltas:
        # apply Delta_i
        H = graphs.PathGraph(i+1)
        H.allow_loops(True)
        H.add_edge(i, i)
        G = G.tensor_product(H)
        G.merge_vertices([x for x in G.vertices() if x[1] == 0])
        G.allow_loops(False)
    output_graph(G)

elif kind == "complete":
    k = int(sys.argv[2])
    G = graphs.CompleteGraph(k)
    output_graph(G)

else:
    raise ValueError("invalid method")
