#include <cstdint>
#include "testlib.h"
#include <vector>
#include <optional>
#include <bit>
#include <cassert>

using namespace std;

typedef long long ll;

struct Test {
  int n, m, p, q;
  vector<int> arr;
};

const int FLAT = 0;
const int RISE = 1;
const int FALL = 2;

struct Phase {
  int l, r, type;
};

// e.g. less<T> creates an _increasing_ stack
template<typename T, typename Compare>
class MonoStack {
  int idx;
  vector<pair<T, int>> stack;
  Compare cmp;

public:
  MonoStack () : idx(0), stack(), cmp() {}
  
  void push (T x) {
    while (!stack.empty() && cmp(x, stack.back().first))
      stack.pop_back();
    
    stack.emplace_back(x, idx);
    idx++;
  }

  // topmost x such that cmp(x, y) holds, and its original index
  optional<pair<T, int>> top (T y) {
    if (stack.empty() || !cmp(stack[0].first, y))
      return {};

    int ret = 0;
    for (int k = 1 << (bit_width(stack.size()) - 1); k != 0; k >>= 1)
      if (ret + k < (int) stack.size() && cmp(stack[ret + k].first, y))
        ret += k;

    return stack[ret];
  }
};

const int INF = 1e9 + 5;

int read_and_check (InStream &in, Test &t) {
  int K = in.readInt(-1, t.n);

  if (K == -1)
    return INF;
  if (K == 0)
    in.quitf(_pe, "K cannot be 0");

  vector<Phase> phases (K);
  int last_r = 0;
  for (int i = 0; i < K; i++) {
    phases[i].l = in.readInt(last_r, last_r, "Li");
    phases[i].r = in.readInt(phases[i].l + 1, t.n, "Ri");
    
    string dir = in.readToken();
    if (dir == "TASANE")
      phases[i].type = FLAT;
    else if (dir == "TOUSEV")
      phases[i].type = RISE;
    else if (dir == "LASKUV")
      phases[i].type = FALL;
    else
      in.quitf(_pe, "direction must be TASANE/TOUSEV/LASKUV");

    if (phases[i].type != FLAT && phases[i].r - phases[i].l < t.m)
      in.quitf(_wa, "falling and rising sections must have length at least M");
    
    last_r = phases[i].r;
  }

  if (phases.back().r != t.n)
    in.quitf(_wa, "last section must end at n");

  for (int i = 0; i < K - 1; i++)
    if (phases[i].type == phases[i + 1].type)
      in.quitf(_wa, "two sections of the same kind must not follow one another");

  for (int i = 0; i < K; i++) {
    int dist = phases[i].r - phases[i].l;
    int height_diff = t.arr[phases[i].r] - t.arr[phases[i].l];
    
    if (phases[i].type == FLAT) 
      if (abs((ll) t.q * height_diff) >= t.p)
        in.quitf(_wa, "the difference between the ends of a flat section must be strictly less than p/q");
    if (phases[i].type == RISE)
      if ((ll) t.q * height_diff < (ll) t.p * dist)
        in.quitf(_wa, "the height difference between the ends of a rising section must be at least p/q per distance");
    if (phases[i].type == FALL)
      if ((ll) t.q * height_diff > -(ll) t.p * dist)
        in.quitf(_wa, "the height difference between the ends of a fallng section must be at least p/q per distance");
  }

  MonoStack<ll, less_equal<ll>> rise_stack;
  MonoStack<ll, greater_equal<ll>> fall_stack;
  int fall_min = 0, rise_min = 0;
  int ptr = 0;
  for (int i = 0; i <= t.n; i++) {
    if (i >= t.m) {
      rise_stack.push((ll) t.q * t.arr[i - t.m] - (ll) t.p * (i - t.m));
      fall_stack.push((ll) t.q * t.arr[i - t.m] + (ll) t.p * (i - t.m));
    }

    // discover the new shortest falling and rising sections
    auto rx = rise_stack.top((ll) t.q * t.arr[i] - (ll) t.p * i);
    auto fx = fall_stack.top((ll) t.q * t.arr[i] + (ll) t.p * i);
    
    // if both are present, the one with higher second component must be processed first
    bool fall_first = false;
    if (rx && fx) {
      assert(rx.value().second != fx.value().second);
      if (fx.value().second > rx.value().second)
        fall_first = true;
    }

    if (fall_first)
      if (fx && fx.value().second >= fall_min)
        rise_min = max(rise_min, fx.value().second + 1);
    
    if (rx && rx.value().second >= rise_min)
      fall_min = max(fall_min, rx.value().second + 1);

    if (!fall_first)
      if (fx && fx.value().second >= fall_min)
        rise_min = max(rise_min, fx.value().second + 1);

    if (ptr != K && phases[ptr].r == i) {
      // validate containment (height differentials are already validated)
      if (phases[ptr].type != RISE && phases[ptr].l < fall_min)
        in.quitf(_wa, "rising subsegment not allowed in falling/flat sections");
      if (phases[ptr].type != FALL && phases[ptr].l < rise_min)
        in.quitf(_wa, "falling subsegment not allowed in rising/flat sections");

      ptr++;
    }
  }

  return K;
}

int main (int argc, char **argv) {
  registerTestlibCmd(argc, argv);

  Test t;
  t.n = inf.readInt();
  t.m = inf.readInt();
  t.p = inf.readInt();
  t.q = inf.readInt();
  
  t.arr = vector<int> (t.n + 1);
  for (int i = 0; i <= t.n; i++)
    t.arr[i] = inf.readInt();

  int ja = read_and_check(ans, t);
  int pa = read_and_check(ouf, t);

  if (ja == INF && pa == INF)
    quitf(_ok, "OK");
  if (ja == INF && pa != INF)
    quitf(_fail, "participant has solution but jury doesn't; please notify someone");
  if (ja != INF && pa == INF)
    quitf(_wa, "jury has solution but participant doesn't");
  if (ja == pa)
    quitf(_ok, "OK");
  if (ja < pa)
    quitf(_wa, "jury has a better solution");
  if (ja > pa)
    quitf(_fail, "jury has a worse solution; please notify someone");
}
