// full solution

#include <iostream>
#include <vector>
#include <optional>
#include <bit>
#include <map>
#include <algorithm>

using namespace std;

typedef long long ll;

const int FLAT = 0;
const int RISE = 1;
const int FALL = 2;

const string names [3] = {"TASANE", "TOUSEV", "LASKUV"};

struct Cell {
  int value, last_i, last_k;
};

bool operator< (Cell p, Cell q) {
  if (p.value != q.value)
    return p.value < q.value;

  return false;
}

struct Phase {
  int start, end, type;
};

template<typename TSeg,
         TSeg (*op)(TSeg, TSeg),
         TSeg (*e)()>
class Segtree {
public:
  int n;
  vector<TSeg> tree;
  vector<int> lfend;
  vector<int> rgend;

  Segtree (int _n, vector<TSeg> init) {
    _n = max(_n, (int) init.size());
    n = 1;
    while (n < _n) {
      n *= 2;
    }

    tree = vector<TSeg> (2 * n, e());
    lfend = vector<int> (2 * n);
    rgend = vector<int> (2 * n);

    for (int i = n; i < 2 * n; i++) {
      if (i - n < (int) init.size()) {
        tree[i] = init[i - n];
      }
      lfend[i] = i - n;
      rgend[i] = i - n;
    }

    for (int i = n - 1; i > 0; i--) {
      lfend[i] = lfend[2 * i];
      rgend[i] = rgend[2 * i + 1];
      tree[i] = op(tree[2 * i], tree[2 * i + 1]);
    }
  }

  Segtree (vector<TSeg> init) : Segtree(0, init) {}
  
  Segtree (int _n) : Segtree(_n, vector<TSeg> (0, e())) {}

  TSeg query (int ql, int qr, int u = 1) {
    ql = max(ql, lfend[u]);
    qr = min(qr, rgend[u]);

    if (ql > qr)
      return e();
    
    if (ql == lfend[u] && qr == rgend[u])
      return tree[u];

    return op(query(ql, qr, 2 * u),
              query(ql, qr, 2 * u + 1));
  }

  void update (int pos, TSeg value) {
    tree[n + pos] = value;
    for (int u = (n + pos) / 2; u != 0; u /= 2)
      tree[u] = op(tree[2 * u], tree[2 * u + 1]);
  }
};

const int INF = 1e9 + 5;
const ll LINF = 1e18 + 2000;

Cell op_min (Cell p, Cell q) {
  return min(p, q);
}

Cell e_min () {
  return Cell {.value = INF, .last_i = -1, .last_k = -1};
}

typedef Segtree<Cell, op_min, e_min> Tree;

template<typename T, typename Compare>
class MonoStack {
  int idx;
  vector<pair<T, int>> stack;
  Compare cmp;

public:
  MonoStack () : idx(0), stack(), cmp() {}
  
  void push (T x) {
    while (!stack.empty() && cmp(x, stack.back().first))
      stack.pop_back();
    
    stack.emplace_back(x, idx);
    idx++;
  }

  // topmost x such that cmp(x, y) holds, and its original index
  optional<pair<T, int>> top (T y) {
    if (stack.empty() || !cmp(stack[0].first, y))
      return {};

    int ret = 0;
    for (int k = 1 << (bit_width(stack.size()) - 1); k != 0; k >>= 1)
      if (ret + k < (int) stack.size() && cmp(stack[ret + k].first, y))
        ret += k;

    return stack[ret];
  }
};

template<typename TKey>
void make_into_compr (map<TKey, int> &compr) {
  int cur = 0;
  for (auto it = compr.begin(); it != compr.end(); it++) {
    it->second = cur;
    cur++;
  }
}

template<typename TKey>
optional<pair<int, int>> get_range (map<TKey, int> &compr, TKey L, TKey R) {
  auto lit = compr.lower_bound(L);
  if (lit == compr.end())
    return {};

  auto rit = compr.upper_bound(R);
  if (rit == compr.begin())
    return {};
  rit--;

  int l = lit->second;
  int r = rit->second;
  if (l > r)
    return {};

  return make_pair(l, r);
}

int main () {
  ios::sync_with_stdio(false);
  cin.tie(0);

  int n, m, p, q;
  cin >> n >> m >> p >> q;

  vector<int> arr (n + 1);
  for (int i = 0; i <= n; i++)
    cin >> arr[i];

  vector<ll> rise_pot (n + 1), fall_pot (n + 1), flat_pot (n + 1);
  for (int i = 0; i <= n; i++) {
    rise_pot[i] = (ll) q * arr[i] - (ll) p * i;
    fall_pot[i] = (ll) q * arr[i] + (ll) p * i;
    flat_pot[i] = (ll) q * arr[i];
  }

  // first, compute for each r, the minimum allowable l (for certain sections)
  vector<int> rise_stop (n + 1, 0), fall_stop (n + 1, 0);
  MonoStack<ll, less_equal<ll>> rise_stack;
  MonoStack<ll, greater_equal<ll>> fall_stack;
  for (int i = 0; i <= n; i++) {
    if (i >= m) {
      rise_stack.push(rise_pot[i - m]);
      fall_stack.push(fall_pot[i - m]);
    }

    if (i != 0) {
      rise_stop[i] = rise_stop[i - 1];
      fall_stop[i] = fall_stop[i - 1];
    }

    // discover the new shortest falling & rising sections
    auto rx = rise_stack.top(rise_pot[i]);
    auto fx = fall_stack.top(fall_pot[i]);

    // if both are present, the one with higher second component must be processed first
    bool fall_first = false;
    if (rx && fx) {
      if (fx.value().second > rx.value().second)
        fall_first = true;
    }

    if (fall_first)
      if (fx && fx.value().second >= rise_stop[i])
        fall_stop[i] = max(fall_stop[i], fx.value().second + 1);

    if (rx && rx.value().second >= fall_stop[i])
      rise_stop[i] = max(rise_stop[i], rx.value().second + 1);

    if (!fall_first)
      if (fx && fx.value().second >= rise_stop[i])
        fall_stop[i] = max(fall_stop[i], fx.value().second + 1);
  }

  map<pair<ll, int>, int> rise_pot_compr, fall_pot_compr, flat_pot_compr;
  for (int i = 0; i <= n; i++) {
    rise_pot_compr[make_pair(rise_pot[i], i)] = 0;
    fall_pot_compr[make_pair(fall_pot[i], i)] = 0;
    flat_pot_compr[make_pair(flat_pot[i], i)] = 0;
  }

  make_into_compr(rise_pot_compr);
  make_into_compr(fall_pot_compr);
  make_into_compr(flat_pot_compr);

  vector<vector<Cell>> dp (n + 1, vector<Cell> (3, Cell {.value = INF, .last_i = -1, .last_k = -1}));
  for (int k = 0; k < 3; k++)
    dp[0][k].value = 0;

  Tree for_flat (n + 1), for_rise (n + 1), for_fall (n + 1);
  int flat_l = 0, flat_r = 0;
  int rise_l = 0, rise_r = 0;
  int fall_l = 0, fall_r = 0; // all of them are [)
  for (int i = 1; i <= n; i++) {
    for_flat.update(flat_pot_compr[make_pair(flat_pot[i - 1], i - 1)],
                    min(Cell {.value = dp[i - 1][RISE].value, .last_i = i - 1, .last_k = RISE},
                        Cell {.value = dp[i - 1][FALL].value, .last_i = i - 1, .last_k = FALL}));
    flat_r = i;
    
    if (i >= m) {
      for_rise.update(rise_pot_compr[make_pair(rise_pot[i - m], i - m)],
                      min(Cell {.value = dp[i - m][FALL].value, .last_i = i - m, .last_k = FALL},
                          Cell {.value = dp[i - m][FLAT].value, .last_i = i - m, .last_k = FLAT}));
      rise_r = i - m + 1;
      
      for_fall.update(fall_pot_compr[make_pair(fall_pot[i - m], i - m)],
                      min(Cell {.value = dp[i - m][FLAT].value, .last_i = i - m, .last_k = FLAT},
                          Cell {.value = dp[i - m][RISE].value, .last_i = i - m, .last_k = RISE}));
      fall_r = i - m + 1;
    }

    while (flat_l < max(rise_stop[i], fall_stop[i]) && flat_l < flat_r) {
      for_flat.update(flat_pot_compr[make_pair(flat_pot[flat_l], flat_l)], e_min());
      flat_l++;
    }

    while (rise_l < fall_stop[i] && rise_l < rise_r) {
      for_rise.update(rise_pot_compr[make_pair(rise_pot[rise_l], rise_l)], e_min());
      rise_l++;
    }

    while (fall_l < rise_stop[i] && fall_l < fall_r) {
      for_fall.update(fall_pot_compr[make_pair(fall_pot[fall_l], fall_l)], e_min());
      fall_l++;
    }

    auto flat_range = get_range(flat_pot_compr,
                                make_pair(flat_pot[i] - p + 1, -1), make_pair(flat_pot[i] + p - 1, n + 1));
    if (flat_range) {
      dp[i][FLAT] = for_flat.query(flat_range.value().first, flat_range.value().second);
      dp[i][FLAT].value++;
    }

    auto fall_range = get_range(fall_pot_compr,
                                make_pair(fall_pot[i], -1), make_pair(LINF, n + 1));
    if (fall_range) {
      dp[i][FALL] = for_fall.query(fall_range.value().first, fall_range.value().second);
      dp[i][FALL].value++;
    }

    auto rise_range = get_range(rise_pot_compr,
                                make_pair(-LINF, -1), make_pair(rise_pot[i], n + 1));
    if (rise_range) {
      dp[i][RISE] = for_rise.query(rise_range.value().first, rise_range.value().second);
      dp[i][RISE].value++;
    }
  }

  Cell best = e_min();
  for (int k = 0; k < 3; k++) {
    best = min(best, Cell {.value = dp[n][k].value, .last_i = n, .last_k = k});
  }

  if (best.value == INF) {
    cout << -1 << '\n';
    return 0;
  }

  vector<Phase> ans;
  int cur_i = best.last_i, cur_k = best.last_k;
  while (cur_i > 0) {
    auto nxt = dp[cur_i][cur_k];
    ans.push_back(Phase {.start = nxt.last_i, .end = cur_i, .type = cur_k});
    cur_i = nxt.last_i;
    cur_k = nxt.last_k;
  }

  reverse(ans.begin(), ans.end());

  cout << (int) ans.size() << '\n';
  for (auto phase : ans) {
    cout << phase.start << " " << phase.end << " " << names[phase.type] << '\n';
  }
}
